<?php
    class vendorfuel_utilities {
        private static $instance = null;
        public static function get_instance(){
            if (self::$instance == null) {
                self::$instance = new vendorfuel_utilities();
            }
            return self::$instance;
        }
        public function build_readable_route($context, $callback) { //build a readable rest route
            return array(
                'methods' => WP_REST_Server::READABLE,
                'callback' => array( $context, $callback ),
                'args' => $context->get_endpoint_args_for_item_schema( false ),
            );
        }
        public function build_editable_route($context, $callback) { //build an editable rest route
            return array(
              'methods' => WP_REST_Server::EDITABLE,
              'callback' => array( $context, $callback ),
              'permission_callback' => array( $context, 'permission_check' ),
              'args' => $context->get_endpoint_args_for_item_schema( false ),
            );
        }
        public function file_check() { //check to ensure the required files and directories exist
            if (!is_dir(VENDORFUEL_CONTENT_DIR)) {
                mkdir(VENDORFUEL_CONTENT_DIR);         
            }
            if(!file_exists(CONTROLLER_CACHE_FILE_DIR)) {
                file_put_contents(CONTROLLER_CACHE_FILE_DIR, '');
            }
            if(!file_exists(DEV_CONTROLLER_CACHE_FILE_DIR)) {
                file_put_contents(DEV_CONTROLLER_CACHE_FILE_DIR, '');
            }
            if(!file_exists(TEMPLATE_CACHE_FILE_DIR)) {
                file_put_contents(TEMPLATE_CACHE_FILE_DIR, '{"branches":{"live":{"templates":[], "map":[]},"dev":{"templates":[], "map":[]}}}');
            }
	        if(!file_exists(TEMPLATE_HISTORY_FILE_DIR)) {
		        file_put_contents(TEMPLATE_HISTORY_FILE_DIR, '{"history":{}}');
	        }
        }
        public function build_app($current_user, $override_branch = null) { //determine which front-end app to build (dev or live)
            $this->file_check();
            $temp_data = $this->get_cache($current_user);
            if (!empty(file_get_contents(TEMPLATE_CACHE_FILE_DIR))) {
                $template_cache = $temp_data['data'];
                if (!empty($template_cache['templates'])) {
                    $branch = $this->ActiveBranch($current_user);
                    return $this->build($template_cache, $branch, $override_branch);
                }
            }
        }
        private function build($template_cache, $branch, $override_branch) { //build front-end app
            if (!is_null($override_branch)) {
                $branch = $override_branch;
            }
            if ($branch == "live") {
               return true;
            } else {
                $app_file = DEV_CONTROLLER_CACHE_FILE_DIR;
                $tempJavascript = "";
            foreach($template_cache['templates'] as $item) {
                if (!empty($item['js'])) {
                    $tempJavascript .= $item['js'] . "\n";
                }
            }
            return rest_ensure_response(file_put_contents($app_file, $tempJavascript));
            }     
        }
		public function get_history($current_user) {
			$this->file_check();
			$is_admin = user_can( $current_user, 'administrator' );
			if ($is_admin) {
				$temp_data = json_decode( file_get_contents( TEMPLATE_HISTORY_FILE_DIR ), true );
				return $temp_data;
			} else {
				return false;
			}
		}
		public function save_history($current_user, $history) {
			$this->file_check();
			$is_admin = user_can( $current_user, 'administrator' );
			if ($is_admin) {
				return file_put_contents( TEMPLATE_HISTORY_FILE_DIR, json_encode( $history ) );
			} else {
				return false;
			}
		}
		public function shorten_history($current_user, $new_max) {
        	$changed = false;
			$history = $this->get_history($current_user);
			foreach ($history as $history_key => $template) {
				foreach($template as $template_key => $edits) {
					if (sizeof($edits) > $new_max) {
						$history[$history_key][$template_key] = array_slice($edits, 0, $new_max);
						$changed = true;
					}
				}
			}
			if ($changed) {
				return rest_ensure_response($this->save_history($current_user, $history));
			} else {
				return false;
			}
			
		}
	    public function get_template_edits($current_user, $template_name) {
		    $history = $this->get_history($current_user);
		    $edits = null;
		    foreach ($history as $history_key => $template) {
			    if ($template[$template_name]) {
				    $edits = $template[$template_name];
			    }
		    }
		    if ($edits) {
		    	$timestamps = array();
		    	foreach($edits as $edit) {
				    array_push($timestamps, $edit['date']);
			    }
		    	return $timestamps;
		    }
		    return false;
	    }
	    public function get_template_edit($current_user, $template_name, $timestamp) {
		    $edits = $this->get_template_edits($current_user, $template_name);
		    foreach ($edits as $edit) {
			    if ($edit['date'] === $timestamp) {
			    	return $edit;
			    }
		    }
		    return false;
	    }
        public function get_cache($current_user, $name = null, $single = false, $override_branch = null) { //retrieve the template cache
            $this->file_check();
            $branch = $this->ActiveBranch($current_user);
            if (!is_null($override_branch)) {
                $branch = $override_branch;
            }
            $temp_data = json_decode(file_get_contents(TEMPLATE_CACHE_FILE_DIR), true);
            $template_data = $temp_data['branches'][$branch];
            if ($single) {
                $template_index = $template_data["map"][$name];
                return $template_data["templates"][$template_index]["content"];
            } else {
                return array('file' => TEMPLATE_CACHE_FILE_DIR . "?nocache=" . time(), 'data' => $template_data);
            }
        }
        public function save_cache($data, $current_user, $override_branch = null) { //save the template cache
            $this->file_check();
            $branch = $this->ActiveBranch($current_user);
            if (!is_null($override_branch)) {
                $branch = $override_branch;
            }
            $template_data = json_decode(file_get_contents(TEMPLATE_CACHE_FILE_DIR), true);
            $cache = array('templates' => $data['templates'], 'map' => $data['map']);
            $template_data['branches'][$branch] = $cache;
            return file_put_contents(TEMPLATE_CACHE_FILE_DIR, json_encode($template_data));
        }
        public function ActiveBranch($current_user) { //determine currently active branch
            $anguledit_options = get_option('vendorfuel-anguledit-settings');
            $dev_branch = $anguledit_options->devBranchActive;
            $is_admin = user_can( $current_user, 'administrator' );
            $branch = 'live';
            if ($dev_branch && $is_admin) {
                $branch = 'dev';
            }
            return $branch;
        }
    }