<?php
    /** 
    Plugin Name: VendorFuel
    Plugin URI: https://vendorfuel.com/
    Description: VendorFuel is a next-generation shopping cart that includes everything you need to start selling products online. All of VendorFuel's powerful e-commerce features are included, for one low monthly price. No endless addons or feature upsells! Enterprise-level eCommerce, at a Small Business Price. Documentation & Support at: https://docs.vendorfuel.com
    Version: 2.9.2
    Author: VendorFuel
    Author URI: https://vendorfuel.com/
    License: GPL3
    License URI: https://www.gnu.org/licenses/gpl.txt
    */
    //STATIC VARIABLES
    $general_settings = get_option('vendorfuel-general-settings');
    $apiUrl = ($general_settings && isset($general_settings->apiUrl))?$general_settings->apiUrl:'https://api.vendorfuel.com/production';
    $apiKey = ($general_settings && isset($general_settings->apiKey))?$general_settings->apiKey:null;
    define( 'VENDORFUEL_API_URL', $apiUrl );
    define( 'VENDORFUEL_API_KEY', $apiKey );
    define( 'VENDORFUEL_JS_VERSION', '2.2.0' );
     //root directory of the vendorfuel plugin
    define( 'VENDORFUEL_ROOT_DIR', plugin_dir_path( __FILE__ ) ); //root directory of the vendorfuel plugin
    define( 'VENDORFUEL_ROOT_URL', plugin_dir_url( __FILE__ ) ); //root url of the vendorfuel plugin
    define( 'VENDORFUEL_CONTENT_DIR', WP_CONTENT_DIR . "/vendorfuel" ); //plugin content directory
	
    //define( 'VENDORFUEL_CONTENT_URL', str_replace('http://', 'https://', WP_CONTENT_URL . "/vendorfuel") ); // plugin content url
	define( 'VENDORFUEL_CONTENT_URL', WP_CONTENT_URL . "/vendorfuel" ); // CAUTION: use this in local dev environment only
	
    define( 'TEMPLATE_CACHE_FILE_DIR', VENDORFUEL_CONTENT_DIR . "/vendorfuel-templates.json" ); //file path to the template cache
	define( 'TEMPLATE_HISTORY_FILE_DIR', VENDORFUEL_CONTENT_DIR . "/vendorfuel-template-history.json" ); //file path to the template cache
    define( 'CONTROLLER_CACHE_FILE', "/vendorfuel-controllers.min.js" ); //controller cache file name
    define( 'CONTROLLER_CACHE_FILE_DIR', VENDORFUEL_CONTENT_DIR . CONTROLLER_CACHE_FILE ); //controller cache directory
	
    define( 'CONTROLLER_CACHE_FILE_URL', str_replace('http://', 'https://', VENDORFUEL_CONTENT_URL . CONTROLLER_CACHE_FILE) ); //controller cache url
	//define( 'CONTROLLER_CACHE_FILE_URL', VENDORFUEL_CONTENT_URL . CONTROLLER_CACHE_FILE ); // CAUTION: use this in local dev environment only
	
    define( 'DEV_CONTROLLER_CACHE_FILE', "/dev-vendorfuel-controllers.js" ); //development branch controller cache file name
    define( 'DEV_CONTROLLER_CACHE_FILE_DIR', VENDORFUEL_CONTENT_DIR . DEV_CONTROLLER_CACHE_FILE ); //development branch controller cache directory
	
    define( 'DEV_CONTROLLER_CACHE_FILE_URL', str_replace('http://', 'https://', VENDORFUEL_CONTENT_URL . DEV_CONTROLLER_CACHE_FILE) ); //development branch controller cache url
	//define( 'DEV_CONTROLLER_CACHE_FILE_URL', VENDORFUEL_CONTENT_URL . DEV_CONTROLLER_CACHE_FILE ); // CAUTION: use this in local dev environment only

    define( 'PAYPAL_CLIENT_ID', get_option('vendorfuel-paypal-id')); 

    //EXTERNAL FFUNCTIONS
    require_once 'includes/vendorfuel_utilities.php'; //various encapsulated utilities
    require_once 'includes/vendorfuel_enqueue.php'; //enqueue all scripts and styles; localize variables to the plugin app
    require_once 'includes/vendorfuel_menu.php'; //plugin wordpress menu
    require_once 'includes/vendorfuel_initialize.php'; //initialize app pages and settings
    require_once 'includes/vendorfuel_api.php'; //defines all wprest api routes
    require_once 'includes/vendorfuel_templates.php'; //update templates, sync template caches, and render template shortcodes
    require_once 'includes/vendorfuel_queue.php'; //manages the async queue background processed
    
    //VENDORFUEL CLASS
    class vendorfuel {
        private $utilities;
        private $queue;

        function __construct() { //instantiate templates and utilities classes
            $this->templates = vendorfuel_templates::get_instance();
            $this->utilities = vendorfuel_utilities::get_instance();
            $this->queue     = vendorfuel_queue::get_instance();
        }

        function vendorfuel_enqueue_plugin($hook) { //enqueue the admin assets
            if( $hook !== 'toplevel_page_vendorfuel' ) { return; }
            $assets = new vendorfuel_enqueue($this->utilities);
            $assets->plugin();
        }

        function vendorfuel_enqueue_frontend($hook) { //enqueue the front-end assets
            $assets = new vendorfuel_enqueue($this->utilities);
            $assets->frontend();
        }

        function vendorfuel_admin_init($hook) { //initialize the plugin
            $vendorfuel = new vendorfuel_initialize($this->utilities);
            $vendorfuel->initialize();
        }

        function vendorfuel_admin_menu($hook) { //admin menu
            $menu = new vendorfuel_admin_page();
            $menu->register_menu();
        }

        function vendorfuel_custom_api($hook) { //register the custom api
            $restApi = new vendorfuel_wprest_api($this->utilities, $this->templates, $this->queue);
            $restApi->register();
        }

        function vendorfuel_templates_init($hook) { //initialize template functionality
            if( $hook !== 'toplevel_page_vendorfuel' ) { return; }
            $templates = new vendorfuel_templates();
            $templates->init();
        }

        function vendorfuel_template_shortcode($params) { //register template shortcode
            $templates = new vendorfuel_templates();
            $content = $templates->shortcode("{$params['name']}");
            
             if(isset($params['product_id'])){
                 return str_replace('ID_OF_PRODUCT',$params['product_id'],$content);
             }

             if(isset($params['cat_id'])){
                return str_replace('ID_OF_CATEGORY',$params['cat_id'],$content);
            }
            return $content;
        }

        function vendorfuel_menu_nav_init(){ //initialize the menu nav template into the wordpress header
            $store = get_option('vendorfuel-store-options');
            if($store->options->{'Load Menu Nav Template'} == true){
                echo do_shortcode( '[vf-template name="menu-nav"]' );
            }
        }

        function vendorfuel_footer_init(){ 
            $store = get_option('vendorfuel-store-options');
            if($store->options->{'Load Footer Template'} == true){
                echo do_shortcode( '[vf-template name="footer"]' );
            }
        }

        function vendorfuel_ccpa_init(){ 
            $store = get_option('vendorfuel-store-options');
            if($store->options->{'Load CCPA Template'} == true){
                echo do_shortcode( '[vf-template name="ccpa"]' );
            }
        }

        function enqueue_paypal(){
            $enabled_payments = get_option('vendorfuel-enabled-payments');
            if(is_array($enabled_payments) && array_key_exists('paypal', $enabled_payments) && array_key_exists('paypal_checkout_enabled', $enabled_payments['paypal'])  && $enabled_payments['paypal']['paypal_checkout_enabled'] ){
                if(PAYPAL_CLIENT_ID != null && PAYPAL_CLIENT_ID != ''){
                    wp_enqueue_script( 'paypal-checkout', 'https://www.paypal.com/sdk/js?currency=USD&client-id='.PAYPAL_CLIENT_ID, array(), null,false );
                }
            }       
        }

        function permalink_template_map_update() {
                $anguledit_settings = get_option('vendorfuel-anguledit-settings');

                if(isset($anguledit_settings->template_map)){
                    foreach ($anguledit_settings->template_map as $slug => $template) {
                        $template->url = get_permalink($template->id);
                        $anguledit_settings->template_map->{$slug} = $template;
                    }
                    update_option('vendorfuel-anguledit-settings', $anguledit_settings);
                }
        }

        function my_prefix_comments_open( $open, $post_id ) {
            $post_type = get_post_type( $post_id );
            // allow comments for built-in "post" post type
            if ( $post_type == 'post'|| $post_type == 'vf-product' || $post_type == 'vf-category' ) {
                return true;
            }
            // disable comments for any other post types
            return false;
        }

        // Our custom post type function
        function create_post_types() {
        
            register_post_type( 'vf-product',
            // CPT Options
                array(
                    'labels' => array(
                        'name' => __( 'VF Product Pages' ),
                        'singular_name' => __( 'VF Product Page' )
                    ),
                    'public' => true,
                    'has_archive' => false,
                    'rewrite' => array('slug' => 'products'),
                    'supports' => array('editor', 'title', 'custom-fields'),
                    'show_in_rest' => true,
                    'capability_type' => 'post'
                )
            );

            register_post_type( 'vf-category',
            // CPT Options
                array(
                    'labels' => array(
                        'name' => __( 'VF Category Pages' ),
                        'singular_name' => __( 'VF Category Page' )
                    ),
                    'public' => true,
                    'has_archive' => false,
                    'rewrite' => array('slug' => 'categories'),
                    'supports' => array('editor', 'title', 'custom-fields'),
                    'show_in_rest' => true,
                    'capability_type' => 'post'
                )
            );
        }

        function get_metadesc() {
            global $post;
            return get_post_meta($post->ID, 'vfmetadesc', true);
        }

        // define the action for register yoast_variable replacments
        function register_custom_yoast_variables() {
            wpseo_register_var_replacement( '%%vfmetadesc%%', array( $this , 'get_metadesc'), 'advanced', 'VF Meta Description Field' );
        }
        // Add action

		function vf_custom_post_types_remove_rss_feed () {
			if ( 'vf-product' == get_post_type() || 'vf-category' == get_post_type() ) {
				remove_action( 'wp_head', 'feed_links_extra', 3 );
				remove_action( 'wp_head', 'feed_links', 2 );
			}  
		}
        
}
    
    $js_app = new vendorfuel(); //instantiate vendorfuel class
    add_action( 'admin_enqueue_scripts', array( $js_app, 'vendorfuel_enqueue_plugin' ) );
    add_action( 'wp_footer', array( $js_app, 'vendorfuel_enqueue_frontend' ) );
    //add_action( 'wp_head', array( $js_app, 'vendorfuel_ccpa_init' ) );
    add_action( 'wp_head', array( $js_app, 'vendorfuel_menu_nav_init' ) );
    add_action( 'wp_footer', array( $js_app, 'vendorfuel_footer_init' ) );
    add_action( 'wp_head', array( $js_app, 'enqueue_paypal' ) );
    add_action( 'admin_init', array( $js_app, 'vendorfuel_admin_init' ) );
    add_action( 'admin_menu', array( $js_app, 'vendorfuel_admin_menu' ) );
    add_action( 'rest_api_init', array( $js_app, 'vendorfuel_custom_api' ) );
    add_action( 'init', array( $js_app, 'create_post_types') );
    add_shortcode( 'vf-template', array( $js_app, 'vendorfuel_template_shortcode' ) );
    add_action( 'permalink_structure_changed', array( $js_app, 'permalink_template_map_update'));
    add_filter( 'comments_open', array( $js_app, 'my_prefix_comments_open'), 10 , 2);
    add_action('the_post', array( $js_app,'vf_custom_post_types_remove_rss_feed'));
    add_action( 'wpseo_register_extra_replacements', array( $js_app,'register_custom_yoast_variables'), 10, 0 ); 
    // Hooking up our function to theme setup
    
    

    // add_action( 'current_screen', 'deactivation_popup' );

    /*
    function deactivation_popup()
    {
        if ( get_current_screen()->id == 'plugins' ) {
            require_once VENDORFUEL_ROOT_DIR . 'includes/popup-deactivate.php';
            
            $popup = new Popup_Deactivate();
            
            if ( !$popup->is_disabled() ) {
                Popup::add_popup( $popup );
            }
        }
    }*/
?>